'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = aframeVideoBillboardComponent;

var _cuid = require('cuid');

var _cuid2 = _interopRequireDefault(_cuid);

var _videoStream = require('./video-stream');

var _videoStream2 = _interopRequireDefault(_videoStream);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var PERMISSION_DENIED_EVENT = 'video-permission-denied';
var PLAY_EVENT = 'video-play';

function createVideoElementAsAsset(id) {

  var video = document.querySelector('#' + id);

  if (!video) {
    video = document.createElement('video');
  }

  video.setAttribute('id', id);
  video.setAttribute('autoplay', true);
  video.setAttribute('src', '');

  var assets = document.querySelector('a-assets');

  if (!assets) {
    assets = document.createElement('a-assets');
    document.querySelector('a-scene').appendChild(assets);
  }

  if (!assets.contains(video)) {
    assets.appendChild(video);
  }

  return video;
}

function shrinkwrapMinDimensions(_ref, _ref2) {
  var minWidth = _ref.width,
      minHeight = _ref.height;
  var width = _ref2.width,
      height = _ref2.height;


  var aspectRatio = width / height;

  // assume width is exact, and height is taller than minHeight
  var shrunkWidth = minWidth;
  var shrunkHeight = shrunkWidth / aspectRatio;

  if (shrunkHeight < minHeight) {
    // our assumption was wrong, so we need to grow the shrunk sizes to make
    // height exact
    shrunkHeight = minHeight;
    shrunkWidth = shrunkHeight * aspectRatio;
  }

  return {
    width: shrunkWidth,
    height: shrunkHeight
  };
}

/**
 * @param aframe {Object} The Aframe instance to register with
 * @param componentName {String} The component name to use. Default: 'video-billboard'
 */
function aframeVideoBillboardComponent(aframe, componentName) {

  /**
   * Draggable component for A-Frame.
   */
  aframe.registerComponent(componentName, {
    schema: {
      /*
       * @param {string} [deviceId=''] - Select the specific device for
       * display. Note that if it is not a valid video device, nothing will be
       * shown.
       */
      deviceId: {
        default: '',
        type: 'string'
      },

      /*
       * @param {number} [minWidth=4] - The minimum width in world-units to
       * display the video at. Video aspect ratio will be preserved.
       */
      minWidth: { default: 4 },

      /*
       * @param {number} [minHeight=3] - The minimum height in world-units to
       * display the video at. Video aspect ratio will be preserved.
       */
      minHeight: { default: 3 }
    },

    setDimentions: function setDimentions() {
      var entityEl = this.el;
      var videoEl = this._videoElement;

      var _shrinkwrapMinDimensi = shrinkwrapMinDimensions({ width: this.data.minWidth, height: this.data.minHeight }, { width: videoEl.videoWidth, height: videoEl.videoHeight }),
          width = _shrinkwrapMinDimensi.width,
          height = _shrinkwrapMinDimensi.height;

      // Set the width and height correctly


      entityEl.setAttribute('width', width);
      entityEl.setAttribute('height', height);
    },


    /**
     * Called once when component is attached. Generally for initial setup.
     */
    init: function init() {},


    /**
     * Called when component is attached and when component data changes.
     * Generally modifies the entity based on the data.
     */
    update: function update(oldData) {
      var _this = this;

      if (aframe.utils.deepEqual(oldData, this.data)) {
        return;
      }

      if (this._videoElement && (oldData.minWidth !== this.data.minWidth || oldData.minHeight !== this.data.minHeight)) {
        this.setDimensions();
      }

      if (oldData.deviceId === this.data.deviceId) {
        return;
      }
      // From hree on, device id has changed

      this.cleanUp();

      this._videoId = (0, _cuid2.default)();
      this._activeVideo = {
        source: null,
        stream: null,
        stop: null,
        pause: null,
        play: null
      };

      (0, _videoStream2.default)(this.data.deviceId).then(function (_ref3) {
        var source = _ref3.source,
            stream = _ref3.stream,
            stop = _ref3.stop,
            play = _ref3.play,
            pause = _ref3.pause;


        _this._activeVideo.soure = source;
        _this._activeVideo.stream = stream;
        _this._activeVideo.stop = stop;
        _this._activeVideo.pause = pause;
        _this._activeVideo.play = play;

        // Creating an aframe asset out of a new video tag
        var videoEl = createVideoElementAsAsset(_this._videoId);
        var entityEl = _this.el;

        var onLoadedMetaData = function onLoadedMetaData(_) {

          // Only want this event listener to execute once
          videoEl.removeEventListener('loadeddata', onLoadedMetaData);

          videoEl.play();

          // Pointing this aframe entity to that video as its source
          entityEl.setAttribute('src', '#' + _this._videoId);

          _this.setDimentions();

          entityEl.emit(PLAY_EVENT, { source: source, stream: stream });
        };

        videoEl.addEventListener('loadeddata', onLoadedMetaData);

        // And starting the video streaming
        videoEl.srcObject = stream;

        _this._videoElement = videoEl;
      }).catch(function (error) {
        // TODO: Check error.name for 'PermissionDeniedError'?
        // https://developer.mozilla.org/en-US/docs/Web/API/MediaDevices/getUserMedia
        _this.el.emit(PERMISSION_DENIED_EVENT, { error: error });
      });
    },
    cleanUp: function cleanUp() {
      if (this._videoElement) {
        this.el.removeAttribute('src');
        this._videoElement.srcObject = null;
        this._videoElement.parentNode.removeChild(this._videoElement);
        this._videoElement = null;
      }

      if (this._activeVideo && this._activeVideo.stop) {
        // stop the stream
        this._activeVideo.stop();
      }

      this._videoId = undefined;
      this._activeVideo = {
        source: null,
        stream: null,
        stop: null,
        pause: null,
        play: null
      };
    },


    /**
     * Called when a component is removed (e.g., via removeAttribute).
     * Generally undoes all modifications to the entity.
     */
    remove: function remove() {
      this.cleanUp();
    },


    /**
     * Called when entity pauses.
     * Use to stop or remove any dynamic or background behavior such as events.
     */
    pause: function pause() {
      if (this._videoElement) {
        this._videoElement.pause();
      }

      if (this._activeVideo && this._activeVideo.pause) {
        this._activeVideo.pause();
      }
    },


    /**
     * Called when entity resumes.
     * Use to continue or add any dynamic or background behavior such as events.
     */
    play: function play() {
      if (this._videoElement) {
        this._videoElement.play();
      }

      if (this._activeVideo && this._activeVideo.play) {
        this._activeVideo.play();
      }
    },
    getDevices: function getDevices() {
      return (0, _videoStream.askPermission)().then(_videoStream.getDevices);
    },
    getActiveDevice: function getActiveDevice() {
      return this.activeVideo.source;
    }
  });
}