'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.askPermission = askPermission;
exports.getDevices = getDevices;
exports.default = getVideoStream;
// for pausing promises when there is an inflight permission request
var askingPermissionPromise = false;
var hasPermission = false;

function getVideoBySource(videoSource) {
  var constraints = {
    video: {
      deviceId: videoSource && videoSource.deviceId ? { exact: videoSource.deviceId } : undefined
    }
  };

  return navigator.mediaDevices.getUserMedia(constraints);
}

function getRearFacingVideoSource() {

  return getDevices().then(function (sources) {

    if (!sources.length) {
      throw new Error('Could not find any video sources');
    }

    var rearVideoSource = void 0;

    sources.some(function (sourceInfo) {
      var labelLower = sourceInfo.label.toLowerCase();
      if (labelLower.indexOf('back') !== -1 || labelLower.indexOf('environment') !== -1 || labelLower.indexOf('rear') !== -1) {
        rearVideoSource = sourceInfo;
        return true;
      }
      return false;
    });

    return rearVideoSource || sources[0];
  });
}

function getSourceByDeviceId(deviceId) {

  return getDevices().then(function (sources) {

    if (!sources.length) {
      throw new Error('Could not find any video sources');
    }

    return sources.filter(function (sourceInfo) {
      return sourceInfo.deviceId === deviceId;
    })[0] || sources[0];
  });
}

function handleError(error) {
  // eslint-disable-next-line no-console
  console.error('navigator.getUserMedia error: ', error);
}

function askPermission() {

  // Already has permission, so resolve immediately
  if (hasPermission) {
    return Promise.resolve();
  }

  // There's already a permission request in flight
  // So return that promise (aka; queue up while we wait)
  if (askingPermissionPromise) {
    return askingPermissionPromise;
  }

  // Trigger an ask for permission dialog
  askingPermissionPromise = getVideoBySource().then(function (_) {
    // Permission received! Nothing in flight, so remove reference to stored
    // promise
    askingPermissionPromise = null;
    hasPermission = true;
  });

  return askingPermissionPromise;
}

function getDevices() {
  return navigator.mediaDevices.enumerateDevices().then(function (sources) {
    return sources.filter(function (source) {
      return source.kind === 'videoinput';
    });
  });
}

/**
 * @param deviceId {String|null} The device ID to get the stream for. If
 * omitted, will attempt to get the rear-facing video stream. If rear-facing
 * video stream not detected, will get the first video stream found.
 *
 * @return {source, stream}. source: MediaDeviceInfo. stream: MediaStream.
 */
function getVideoStream(deviceId) {

  var streamPromise = askPermission();

  if (deviceId) {
    streamPromise = streamPromise.then(function (_) {
      return getSourceByDeviceId(deviceId);
    });
  } else {
    streamPromise = streamPromise.then(getRearFacingVideoSource);
  }

  return streamPromise.then(function (source) {
    return getVideoBySource(source).then(function (stream) {
      return {
        source: source,
        stream: stream,
        stop: function stop(_) {
          return stream.getTracks().forEach(function (track) {
            return track.stop();
          });
        },
        pause: function pause(_) {
          return stream.getTracks().forEach(function (track) {
            track.enabled = false;
          });
        },
        play: function play(_) {
          return stream.getTracks().forEach(function (track) {
            track.enabled = true;
          });
        }
      };
    });
  }).catch(handleError);
}